//Code by Chavez Clemons Jr, July 6, 2012 for CSC2201

#include "QueueArray.h"

template <typename DataType>
QueueArray<DataType>::QueueArray(int maxNumber) //QueueArray default constructor; creates a new dynamic array,
{												//initalizes it and maxSize to maxNumber, and sets front and back
	dataItems = new DataType[maxNumber];		//equal to -1
	maxSize = maxNumber;
	front=back=-1;
	for (int i = 0; i < maxSize; i++)
		dataItems[i] = NULL;
}

template <typename DataType>
QueueArray<DataType>::QueueArray(const QueueArray& other) //QueueArray copy constuctor; initalizes current QueueArray
{														  //to the other QueueArray's information
	maxSize = other.maxSize;
	dataItems = new DataType [maxSize];
	front = other.front;
	back = other.back;
	for (int i = 0; i < maxSize; i++)
	{
		dataItems[i] = other.dataItems[i];
	}
}

template <typename DataType>
QueueArray<DataType>& QueueArray<DataType>::operator=(const QueueArray& other) //Same as copy constructor, but checks to make sure 
{																			   //both sides are different and returns the result
	if (this != &other) //Only perform these actions if the two sides of the '=' operation are different
	{
		delete [] dataItems; //Delete the current dataItems object and replace the contents with the input dataItems object
		maxSize = other.maxSize;
		dataItems = new DataType [maxSize];
		front = other.front;
		back = other.back;
		for (int i = 0; i < maxSize; i++)
		{
			dataItems[i] = other.dataItems[i];
		}
	}
}

template <typename DataType>
QueueArray<DataType>::~QueueArray() //Destructor; calls clear() function
{
	if(!isEmpty())
	{
		clear();
	}
}


template <typename DataType>
void QueueArray<DataType>::enqueue(const DataType& newDataItem) throw (logic_error) //Enqueue function; inserts into the queue with 4 cases
{
	if (isEmpty()) //Case 1: if array is empty, increment front and back to 0 and set dataItems[0] equal to the input
	{
		front++;
		back++;
		dataItems[back] = newDataItem;
	}

	else if (!isFull() && back+1 != maxSize) //Case 2: if array isn't full and back incremented wouldn't equal the maxSize,
	{										//increment back and set that index value equal to the input value
			back++;
			dataItems[back] = newDataItem;
	}
	else if (!isFull() && back+1 == maxSize) //Case 3: if array isn't full and back incremented would equal the maxSize,
	{										//set back equal to zero and set that index value equal to the input value
		back = 0;
		dataItems[back] = newDataItem;
	}
	else //Case 4: if array is full, throw an error
	{
		throw logic_error("Queue is full.");
	}
}

template <typename DataType>
DataType QueueArray<DataType>::dequeue() throw (logic_error) //Dequeue function; removes from the queue
{
	DataType temp; //Create a temp data variable
	if (!isEmpty()) //If the queue isn't empty,
	{
		temp = dataItems[front]; //Set the temp variable equal to the value of the dataItem at index 'front', 
		dataItems[front] = NULL; //set front equal to NULL
		if (front == back) //If front equals the back when the dataItem is deleted, clear the array,
			clear();
		else if (front+1 == maxSize) //Else if the front incremented would equal the maxSize, set it equal to zero
			front = 0;
		else //Otherwise, increment front
			front++;
		return temp; //Return the value of the temporary variable
	}
	else //If the queue is empty, throw an error
	{
		throw logic_error("Queue is empty.");
		return 0;
	}
}

template <typename DataType>
void QueueArray<DataType>::clear() //Clear function
{
	if (!isEmpty()) //If the queue isn't empty, delete the array, create an empty array and initalize front & back to -1
	{
		delete [] dataItems;
		dataItems = new DataType [maxSize];
		front=back=-1;
		for (int i = 0; i < maxSize; i++)
			dataItems[i] = NULL;
	}
}

template <typename DataType>
bool QueueArray<DataType>::isEmpty() const //isEmpty function
{
	if (front==back && back == -1) //If both the front and back equal -1, the array is empty
		return true;
	return false; //Otherwise it isn't
}

template <typename DataType>
bool QueueArray<DataType>::isFull() const //isFull function
{ //If we can't wrap around to the beginning of the queue or back would equal front with there being an existing data item, the queue is full
	if ((back+1 == maxSize && dataItems[0] != NULL) || (back+1 == front && dataItems[front] != NULL))
		return true;
	return false; //Otherwise it isn't
}

template <typename DataType>
void QueueArray<DataType>::putFront(const DataType& newDataItem) throw (logic_error)
{
}

template <typename DataType>
DataType QueueArray<DataType>::getRear() throw (logic_error)
{
	DataType temp;
	return temp;
}

template <typename DataType>
int QueueArray<DataType>::getLength() const //getLength function
{
	int count = 0; //Iterates through the array checking to see if there is a value other than NULL for each index
	for (int i = 0; i < maxSize; i++)
	{
		if (dataItems[i] != NULL)
			count++;
	}
	return count; //Returns the number of non NULL values
}

//--------------------------------------------------------------------

template <typename DataType>
void QueueArray<DataType>::showStructure() const 
// Array implementation. Outputs the data items in a queue. If the
// queue is empty, outputs "Empty queue". This operation is intended
// for testing and debugging purposes only.

{
    int j;   // Loop counter

    if ( front == -1 )
       cout << "Empty queue" << endl;
    else
    {
       cout << "Front = " << front << "  Back = " << back << endl;
       for ( j = 0 ; j < maxSize ; j++ )
           cout << j << "\t";
       cout << endl;
       if ( back >= front )
          for ( j = 0 ; j < maxSize ; j++ )
              if ( ( j >= front ) && ( j <= back ) )
                 cout << dataItems[j] << "\t";
              else
                 cout << " \t";
       else
          for ( j = 0 ; j < maxSize ; j++ )
              if ( ( j >= front ) || ( j <= back ) )
                 cout << dataItems[j] << "\t";
              else
                 cout << " \t";
       cout << endl;
    }
}